/*
 * Developed by Hillstone Software. http://www.hillstone-software.com
 * TCP module
 */

#ifndef _HS_TCP_IF_H
#define _HS_TCP_IF_H

#define HS_PROTOCOL_TCP 6			// IP port number used for TCP

// API return codes
typedef enum
{
	HS_TCP_RC_OK,					// Success
	HS_TCP_RC_ALR_INIT,				// Already intiialized
	HS_TCP_RC_NOT_INIT,				// Not initialized
	HS_TCP_RC_BAD_PARAM,			// Invalid parameter(s)
	HS_TCP_RC_NO_MEM,				// not enough memory
	HS_TCP_RC_NO_CTX,				// no free contexts
	HS_TCP_RC_TXQ_FULL,				// transmit queue is full, cannot send queue buffer
	HS_TCP_RC_INV_STATE,			// Requested action is invalid in current session state
} hstcp_rc_t;

// user events
typedef enum
{
	HS_TCP_USR_EV_CONNECT,			// Incoming TCP connect received, user must accept or reject
	HS_TCP_USR_EV_DATA,				// Data received on TCP connection
	HS_TCP_USR_EV_RELEASE,			// TCP session closed
	HS_TCP_USR_EV_ACCEPT,			// TCP connection accepted by remote peer
} hstcp_usr_event_t;

// callback used by HSTCP to get number of millisecond ticks since bootup
// Used for generating pseudo random initial sequence number
// if callback pointer not initialized, HSTCP will use default 
// pseudo random number generator to produce initial sequence numbers
typedef unsigned long hs_tcp_get_ticks_fn_t(void);

// timer callback prototype
typedef void hs_tcp_timer_cb_t(void *arg);

// callback used by HSTCP to start a timer
typedef long hs_tcp_timer_start_t(unsigned long timeout_ms, void *arg, hs_tcp_timer_cb_t *cb);

// callback used by HSTCP to stop a timer
typedef void hs_tcp_timer_stop_t(long timer_han);

// get buffer for received data callback
typedef unsigned char *hs_tcp_get_buf_t(long user_handle, unsigned short int length);

// event log function
typedef void hs_tcp_log_fn_t(char *str);

typedef struct 
{
	unsigned long	local_ip;				// local IP address
	int				max_sessions;			// maximum number of TCP sessions
	unsigned long	window_size;			// TCP window size in bytes
	
	unsigned long	max_retrans_timeout;	// maximum retransmission timeout in milliseconds
	unsigned long	min_retrans_timeout;	// minimum retransmission timeout in milliseconds
	unsigned short	data_tx_attempts;		// number of retransmission attempts before giving up

	unsigned long	connect_timeout;		// SYN retransmit timeout in milliseconds
	unsigned short	connect_retries;		// number of times to retransmit SYN before giving up

	unsigned long	wait_state_timeout;		// time in milliseconds to wait in TCP time wait state before releasing context



	hs_tcp_get_ticks_fn_t *get_ticks_fn;	// callback used by HSTCP to get number of millisecond ticks since bootup
											// Used for generating pseudo random initial sequence number
											// if callback pointer not initialized, HSTCP will use default PRNG
											// to produce initial sequence numbers

	unsigned long	seed;					// initial seed used by HSTCP to initialize PRNG for producing initial sequece numbers

	hs_tcp_timer_start_t	*start_timer;	// timer start callback
	hs_tcp_timer_stop_t 	*stop_timer;	// timer stop callback
	hs_tcp_get_buf_t 		*get_buffer;	// get buffer callback
	hs_tcp_log_fn_t			*log_fn;		// event log function

	unsigned short int txq_size;			// size of transmit queue in bytes
} hstcp_init_t;

// tcp session callback
typedef void hs_tcp_event_fn_t(long user_handle, int ev, long arg1, long arg2, long arg3);

typedef struct
{
	unsigned long	local_ip;				// local ip address
} hs_tcp_params_t;

/*
 * Initialise TCP protocol module
 */
extern int HsTcpInit(hstcp_init_t *init);

/*
 * Set parameters at run time
 */
extern void HsTcpSetParams(hs_tcp_params_t *pParams);

/*
 * Shutdown TCP stack
 */
extern void HsTcpCleanUp(void);

/*
 * Create outgoing TCP session
 */
extern
int HsTcpConnect(long 					usr_handle, 		// user handle
					unsigned long		dest_ip,			// destination IP address to connect to
					unsigned short		source_port,		// source port
					unsigned short		dest_port,			// destination port
					hs_tcp_event_fn_t	*event_callback,	// event callback
					long 				*tcp_handle);		// TCP new session handle

/*
 * Start listening for incoming TCP connections on specified port from specified IP. Set IP to zero to take connections from any IP
 */
extern
int HsTcpListen(long usr_handle, unsigned short port, unsigned long remote_ip, 
	hs_tcp_event_fn_t *event_callback, long *listen_handle);

/*
 * Bind user context to a socket context
 */
int HsTcpBindSession(long conn_handle, long user_han);

/*
 * Stop listening for incoming TCP connections
 */
extern
int HsTcpStopListen(long listen_handle);

/*
 * Close TCP session
 */
extern int HsTcpClose(long tcp_handle);

/*
 * Send packet over TCP session
 */
extern
int HsTcpSend(long				conn_handle,	// TCP connection handle
				unsigned char 	*buffer, 		// packet data
				int 			length,         // packet data length
				int             *txed_len);		// bytes actually transmitted

/*
 * TCP packet received from IP layer
 */ 
extern void HsTcpReceivePacket(unsigned char *pTcpPkt, unsigned short tcp_len);


#endif // _HS_TCP_IF_H
